/* $Id: check_unixio.c 659 2006-05-13 14:51:08Z jim $
   teebu - An archiving tool
   Copyright (C) 2006 Jim Farrand

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>

#include "check_io.h"
#include "check_unixio.h"

#include "unixio.h"

#define TEST_FILE "/tmp/bumtest.baseio.tmp"

static int
read_write_test_size ()
{
  switch (get_test_size ())
    {
    case QUICK_TEST:
      return 256;
    case NORMAL_TEST:
      return 1024;
    case LONG_TEST:
      return 10240;
    default:
      fail ("Invalid test size");
      return -1;                // Not reached, prevent compiler warning
    }
}

START_TEST (check_out_basic)
{
  out_stream_t outs =
    unixio_open_out (TEST_FILE, O_WRONLY | O_CREAT | O_TRUNC, 0644);
  basic_out_test (outs);
  release_out (outs);
}
END_TEST static void
check_read_write (int block_size)
{
  const int
    test_size = read_write_test_size (), count = test_size / block_size;

  assert (count > 0);
  assert (test_size > 0);

  out_stream_t outs =
    unixio_open_out (TEST_FILE, O_WRONLY | O_CREAT | O_TRUNC, 0644);
  ascending_output_half_test (false, outs, block_size, count);
  release_out (outs);

  mark_point ();

  in_stream_t ins = unixio_open_in (TEST_FILE, O_RDONLY);
  ascending_input_half_test (true, false, ins, block_size, count);
  release_in (ins);
}

START_TEST (check_read_write_small_blocks)
{
  check_read_write (1);
}

END_TEST
START_TEST (check_read_write_medium_blocks)
{
  check_read_write (32);
}

END_TEST
START_TEST (check_read_write_large_blocks)
{
  check_read_write (256);
}
END_TEST void
add_unixio_tests (Suite * s)
{
  TCase *tc_core = tcase_create ("UnixIO");
  tcase_add_test (tc_core, check_out_basic);
  tcase_add_test (tc_core, check_read_write_small_blocks);
  tcase_add_test (tc_core, check_read_write_medium_blocks);
  tcase_add_test (tc_core, check_read_write_large_blocks);

  suite_add_tcase (s, tc_core);
}
